/*
 *
 *    Copyright (C) 2004 NERC DataGrid
 *    This software may be distributed under the terms of the
 *    CCLRC Licence for CCLRC Software
 * <CDATDIR>/External_License/CCLRC_CDAT_License.txt 
 *
 */
/* including this header file will rename all the functions in the library
 * used for WGDOS unpacking within cdunifpp, in order to avoid potential
 * namespace conflicts 
 */

#define bit_test pp_unwgdos__bit_test
#define extrin pp_unwgdos__extrin
#define move_bits pp_unwgdos__move_bits
#define read_wgdos_header pp_unwgdos__read_wgdos_header
#define unwgdos pp_unwgdos__unwgdos
#define xpnd pp_unwgdos__xpnd
#define c8_to_i4 pp_unwgdos__c8_to_i4
#define c8toi4_ pp_unwgdos__c8toi4_
#define c8_to_i8 pp_unwgdos__c8_to_i8
#define c8toi8_ pp_unwgdos__c8toi8_
#define c8_to_l4 pp_unwgdos__c8_to_l4
#define c8tol4_ pp_unwgdos__c8tol4_
#define c8_to_l8 pp_unwgdos__c8_to_l8
#define c8tol8_ pp_unwgdos__c8tol8_
#define c8_to_r4 pp_unwgdos__c8_to_r4
#define c8tor4_ pp_unwgdos__c8tor4_
#define c8_to_r8 pp_unwgdos__c8_to_r8
#define c8tor8_ pp_unwgdos__c8tor8_
#define cos_backspace pp_unwgdos__cos_backspace
#define cosbackspace_ pp_unwgdos__cosbackspace_
#define cos_close pp_unwgdos__cos_close
#define cosclose_ pp_unwgdos__cosclose_
#define cos_open pp_unwgdos__cos_open
#define cosopen_ pp_unwgdos__cosopen_
#define cos_read pp_unwgdos__cos_read
#define cosread_ pp_unwgdos__cosread_
#define cos_rewind pp_unwgdos__cos_rewind
#define cosrewind_ pp_unwgdos__cosrewind_
#define expand21_ pp_unwgdos__expand21_
#define expand21_r4_ pp_unwgdos__expand21_r4_
#define expand21_r8_ pp_unwgdos__expand21_r8_
#define i4_to_i8 pp_unwgdos__i4_to_i8
#define i8_to_i4 pp_unwgdos__i8_to_i4
#define ibmi2_to_i4 pp_unwgdos__ibmi2_to_i4
#define ibmi2toi4_ pp_unwgdos__ibmi2toi4_
#define ibmi2_to_i8 pp_unwgdos__ibmi2_to_i8
#define ibmi2toi8_ pp_unwgdos__ibmi2toi8_
#define ibmi4_to_i4 pp_unwgdos__ibmi4_to_i4
#define ibmi4toi4_ pp_unwgdos__ibmi4toi4_
#define ibmi4_to_i8 pp_unwgdos__ibmi4_to_i8
#define ibmi4toi8_ pp_unwgdos__ibmi4toi8_
#define ibmr4_to_r4 pp_unwgdos__ibmr4_to_r4
#define ibmr4tor4_ pp_unwgdos__ibmr4tor4_
#define ibmr4_to_r8 pp_unwgdos__ibmr4_to_r8
#define ibmr4tor8_ pp_unwgdos__ibmr4tor8_
#define ibmr8_to_r8 pp_unwgdos__ibmr8_to_r8
#define ibmr8tor8_ pp_unwgdos__ibmr8tor8_
#define ii4toi8_ pp_unwgdos__ii4toi8_
#define ii8toi4_ pp_unwgdos__ii8toi4_
#define r4_to_r8 pp_unwgdos__r4_to_r8
#define r4tor8_ pp_unwgdos__r4tor8_
#define r8_to_r4 pp_unwgdos__r8_to_r4
#define r8tor4_ pp_unwgdos__r8tor4_
#define swap_bytes pp_unwgdos__swap_bytes
#define swapbytes_ pp_unwgdos__swapbytes_
